"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.splitLiteral = exports.SetupData = exports.PortBrand = exports.Port = exports.IPv6FromString = exports.IPv6Endpoint = exports.IPv6CidrMaskBrand = exports.IPv6CidrMask = exports.IPv6 = exports.IPv4FromString = exports.IPv4Endpoint = exports.IPv4CidrMaskBrand = exports.IPv4CidrMask = exports.IPv4 = exports.HostnameEndpoint = exports.Endpoint = exports.DurationFromSecondsString = exports.DurationFromSecondsBrand = exports.DurationFromSeconds = exports.CidrBlockFromString = exports.CidrBlock = exports.AddressFromString = exports.Address = void 0;
var ParseResult = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/schema/ParseResult"));
var Schema = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/schema/Schema"));
var Array = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Array"));
var Brand = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Brand"));
var Duration = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Duration"));
var Function = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Function"));
var Option = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Option"));
var Predicate = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Predicate"));
var Stream = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Stream"));
var String = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/String"));
var net = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("node:net"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/**
 * Internet schemas for wireguard configuration.
 *
 * @since 1.0.0
 */

/** @since 1.0.0 */
const splitLiteral = (str, delimiter) => str.split(delimiter);
/**
 * @since 1.0.0
 * @category Branded constructors
 */
exports.splitLiteral = splitLiteral;
const DurationFromSecondsBrand = exports.DurationFromSecondsBrand = /*#__PURE__*/Brand.nominal();
/**
 * Transforms a `number` of seconds into a `Duration`.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Duration from "effect/Duration";
 *     import * as Schema from "@effect/schema/Schema";
 *     import {
 *         DurationFromSeconds,
 *         DurationFromSecondsBrand,
 *     } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeDuration = Schema.decodeSync(DurationFromSeconds);
 *     const duration = decodeDuration(11);
 *     assert.strictEqual(Duration.toSeconds(duration), 11);
 */
const DurationFromSeconds = exports.DurationFromSeconds = /*#__PURE__*/Schema.transform(Schema.Int, Schema.DurationFromSelf, {
  decode: seconds => Duration.seconds(seconds),
  encode: duration => Duration.toSeconds(duration)
}).pipe(Schema.fromBrand(DurationFromSecondsBrand)).annotations({
  identifier: "DurationFromSeconds",
  description: "A duration from a number of seconds"
});
/**
 * Transforms a `string` of seconds into a `Duration`.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Duration from "effect/Duration";
 *     import * as Schema from "@effect/schema/Schema";
 *     import { DurationFromSecondsString } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeDurationString = Schema.decodeSync(
 *         DurationFromSecondsString
 *     );
 *     const duration = decodeDurationString("12");
 *     assert.strictEqual(Duration.toSeconds(duration), 12);
 */
const DurationFromSecondsString = exports.DurationFromSecondsString = /*#__PURE__*/Schema.transform(Schema.NumberFromString, DurationFromSeconds, {
  decode: Function.identity,
  encode: Function.identity
}).annotations({
  identifier: "DurationFromSecondsString",
  description: "A duration from a string of seconds"
});
/**
 * @since 1.0.0
 * @category Branded constructors
 */
const PortBrand = exports.PortBrand = /*#__PURE__*/Brand.nominal();
/**
 * An operating system port number.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { Port, PortBrand } from "the-wireguard-effect/InternetSchemas";
 *
 *     const port: PortBrand = PortBrand(8080);
 *     assert.strictEqual(port, 8080);
 *
 *     const decodePort = Schema.decodeSync(Port);
 *     assert.strictEqual(decodePort(8080), 8080);
 *
 *     assert.throws(() => decodePort(65536));
 *     assert.doesNotThrow(() => decodePort(8080));
 */
const Port = exports.Port = /*#__PURE__*/Schema.Int.pipe(Schema.between(0, 2 ** 16 - 1)).pipe(Schema.fromBrand(PortBrand)).annotations({
  identifier: "Port",
  description: "A port number"
});
/**
 * An IPv4 address.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { IPv4 } from "the-wireguard-effect/InternetSchemas";
 *
 *     const ipv4: IPv4 = new IPv4({ ip: "1.1.1.1" });
 *     assert.strictEqual(ipv4, "1.1.1.1");
 *
 *     const decodeIPv4 = Schema.decodeSync(IPv4);
 *     assert.strictEqual(decodeIPv4({ ip: "1.1.1.1" }), "1.1.1.1");
 *
 *     assert.throws(() => decodeIPv4({ ip: "1.1.a.1" }));
 *     assert.doesNotThrow(() => decodeIPv4({ ip: "1.1.1.1" }));
 */
class IPv4 extends Schema.Class("IPv4")({
  ip: Schema.String.pipe(Schema.filter((s, _, ast) => net.isIPv4(s) ? Option.none() : Option.some(new ParseResult.Type(ast, s))))
}) {
  /** @since 1.0.0 */
  family = "ipv4";
  /** @since 1.0.0 */
  static FromBigint = n => {
    const padded = n.toString(16).replace(/:/g, "").padStart(8, "0");
    const groups = [];
    for (let i = 0; i < 8; i += 2) {
      const h = padded.slice(i, i + 2);
      groups.push(parseInt(h, 16));
    }
    return Schema.decode(IPv4)({
      ip: groups.join(".")
    });
  };
  /** @since 1.0.0 */
  get asBigint() {
    return Function.pipe(this.ip, String.split("."), Array.map(s => Number.parseInt(s, 10)), Array.map(n => n.toString(16)), Array.map(String.padStart(2, "0")), Array.join(""), hex => BigInt(`0x${hex}`));
  }
}
/**
 * A schema that transforms a `string` into an `IPv4`.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { IPv4FromString } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeIPv4 = Schema.decodeSync(IPv4FromString);
 *     assert.strictEqual(decodeIPv4("1.1.1.1").ip, "1.1.1.1");
 *
 *     assert.throws(() => decodeIPv4("1.1.a.1"));
 *     assert.doesNotThrow(() => decodeIPv4("1.1.1.1"));
 *
 * @see {@link IPv4}
 */
exports.IPv4 = IPv4;
const IPv4FromString = exports.IPv4FromString = /*#__PURE__*/Schema.transform(Schema.String, IPv4, {
  decode: str => ({
    ip: str
  }),
  encode: ({
    ip
  }) => ip
}).annotations({
  identifier: "IPv4FromString",
  description: "An ipv4 address"
});
/**
 * An IPv6 address.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { IPv6 } from "the-wireguard-effect/InternetSchemas";
 *
 *     const ipv6: IPv6 = new IPv6({
 *         ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
 *     });
 *     assert.strictEqual(ipv6, "2001:0db8:85a3:0000:0000:8a2e:0370:7334");
 *
 *     const decodeIPv6 = Schema.decodeSync(IPv6);
 *     assert.deepStrictEqual(
 *         decodeIPv6({ ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334" }),
 *         "2001:0db8:85a3:0000:0000:8a2e:0370:7334"
 *     );
 *
 *     assert.throws(() =>
 *         decodeIPv6({ ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334:" })
 *     );
 *     assert.throws(() => decodeIPv6({ ip: "2001::85a3::0000::0370:7334" }));
 *     assert.doesNotThrow(() =>
 *         decodeIPv6({ ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334" })
 *     );
 */
class IPv6 extends Schema.Class("IPv6")({
  ip: Schema.String.pipe(Schema.filter((s, _, ast) => net.isIPv6(s) ? Option.none() : Option.some(new ParseResult.Type(ast, s))))
}) {
  /** @since 1.0.0 */
  family = "ipv6";
  /** @since 1.0.0 */
  static FromBigint = n => {
    const hex = n.toString(16).padStart(32, "0");
    const groups = [];
    for (let i = 0; i < 8; i++) {
      groups.push(hex.slice(i * 4, (i + 1) * 4));
    }
    return Schema.decode(IPv6)({
      ip: groups.join(":")
    });
  };
  /** @since 1.0.0 */
  get asBigint() {
    function paddedHex(octet) {
      return parseInt(octet, 16).toString(16).padStart(4, "0");
    }
    let groups = [];
    const halves = this.ip.split("::");
    if (halves.length === 2) {
      let first = halves[0].split(":");
      let last = halves[1].split(":");
      if (first.length === 1 && first[0] === "") {
        first = [];
      }
      if (last.length === 1 && last[0] === "") {
        last = [];
      }
      const remaining = 8 - (first.length + last.length);
      if (!remaining) {
        throw new Error("Error parsing groups");
      }
      groups = groups.concat(first);
      for (let i = 0; i < remaining; i++) {
        groups.push("0");
      }
      groups = groups.concat(last);
    } else if (halves.length === 1) {
      groups = this.ip.split(":");
    } else {
      throw new Error("Too many :: groups found");
    }
    groups = groups.map(group => parseInt(group, 16).toString(16));
    if (groups.length !== 8) {
      throw new Error("Invalid number of groups");
    }
    return BigInt(`0x${groups.map(paddedHex).join("")}`);
  }
}
/**
 * A schema that transforms a `string` into an `IPv6`.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { IPv6FromString } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeIPv6 = Schema.decodeSync(IPv6FromString);
 *     assert.deepStrictEqual(
 *         decodeIPv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334").ip,
 *         "2001:0db8:85a3:0000:0000:8a2e:0370:7334"
 *     );
 *
 *     assert.throws(() =>
 *         decodeIPv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334:")
 *     );
 *     assert.throws(() => decodeIPv6("2001::85a3::0000::0370:7334"));
 *     assert.doesNotThrow(() =>
 *         decodeIPv6("2001:0db8:85a3:0000:0000:8a2e:0370:7334")
 *     );
 *
 * @see {@link IPv6}
 */
exports.IPv6 = IPv6;
const IPv6FromString = exports.IPv6FromString = /*#__PURE__*/Schema.transform(Schema.String, IPv6, {
  decode: str => ({
    ip: str
  }),
  encode: ({
    ip
  }) => ip
}).annotations({
  identifier: "IPv6FromString",
  description: "An ipv6 address"
});
/**
 * An IP address, which is either an IPv4 or IPv6 address.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { Address } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeAddress = Schema.decodeSync(Address);
 *
 *     assert.throws(() => decodeAddress({ ip: "1.1.b.1" }));
 *     assert.throws(() =>
 *         decodeAddress({ ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334:" })
 *     );
 *
 *     assert.doesNotThrow(() => decodeAddress({ ip: "1.1.1.2" }));
 *     assert.doesNotThrow(() =>
 *         decodeAddress({ ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334" })
 *     );
 *
 * @see {@link IPv4}
 * @see {@link IPv6}
 */
const Address = exports.Address = /*#__PURE__*/Schema.Union(IPv4, IPv6).annotations({
  identifier: "Address",
  description: "An ip address"
});
/**
 * An IP address, which is either an IPv4 or IPv6 address. This schema
 * transforms a `string` into an `Address`.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { AddressFromString } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeAddress = Schema.decodeSync(AddressFromString);
 *
 *     assert.throws(() => decodeAddress("1.1.b.1"));
 *     assert.throws(() =>
 *         decodeAddress("2001:0db8:85a3:0000:0000:8a2e:0370:7334:")
 *     );
 *
 *     assert.doesNotThrow(() => decodeAddress("1.1.1.2"));
 *     assert.doesNotThrow(() =>
 *         decodeAddress("2001:0db8:85a3:0000:0000:8a2e:0370:7334")
 *     );
 */
const AddressFromString = exports.AddressFromString = /*#__PURE__*/Schema.Union(IPv4FromString, IPv6FromString).annotations({
  identifier: "AddressFromString",
  description: "An ip address"
});
/**
 * @since 1.0.0
 * @category Branded constructors
 */
const IPv4CidrMaskBrand = exports.IPv4CidrMaskBrand = /*#__PURE__*/Brand.nominal();
/**
 * An ipv4 cidr mask, which is a number between 0 and 32.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import {
 *         IPv4CidrMask,
 *         IPv4CidrMaskBrand,
 *     } from "the-wireguard-effect/InternetSchemas";
 *
 *     const mask: IPv4CidrMaskBrand = IPv4CidrMaskBrand(24);
 *     assert.strictEqual(mask, 24);
 *
 *     const decodeMask = Schema.decodeSync(IPv4CidrMask);
 *     assert.strictEqual(decodeMask(24), 24);
 *
 *     assert.throws(() => decodeMask(33));
 *     assert.doesNotThrow(() => decodeMask(0));
 *     assert.doesNotThrow(() => decodeMask(32));
 */
const IPv4CidrMask = exports.IPv4CidrMask = /*#__PURE__*/Schema.Int.pipe(Schema.between(0, 32)).pipe(Schema.fromBrand(IPv4CidrMaskBrand)).annotations({
  identifier: "IPv4CidrMask",
  description: "An ipv4 cidr mask"
});
/**
 * @since 1.0.0
 * @category Branded constructors
 */
const IPv6CidrMaskBrand = exports.IPv6CidrMaskBrand = /*#__PURE__*/Brand.nominal();
/**
 * An ipv6 cidr mask, which is a number between 0 and 128.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import {
 *         IPv6CidrMask,
 *         IPv6CidrMaskBrand,
 *     } from "the-wireguard-effect/InternetSchemas";
 *
 *     const mask: IPv6CidrMaskBrand = IPv6CidrMaskBrand(64);
 *     assert.strictEqual(mask, 64);
 *
 *     const decodeMask = Schema.decodeSync(IPv6CidrMask);
 *     assert.strictEqual(decodeMask(64), 64);
 *
 *     assert.throws(() => decodeMask(129));
 *     assert.doesNotThrow(() => decodeMask(0));
 *     assert.doesNotThrow(() => decodeMask(128));
 */
const IPv6CidrMask = exports.IPv6CidrMask = /*#__PURE__*/Schema.Int.pipe(Schema.between(0, 128)).pipe(Schema.fromBrand(IPv6CidrMaskBrand)).annotations({
  identifier: "IPv6CidrMask",
  description: "An ipv6 cidr mask"
});
/**
 * Internal helper representation of a cidr range so that we can do
 * transformations to and from it as we cannot represent a stream using
 *
 * @since 1.0.0
 * @category Schemas
 */
class CidrBlock extends Schema.Class("CidrBlock")({
  /** The IP address of this cidr block. */
  ip: AddressFromString,
  /** The subnet mask of this cidr block. */
  mask: Schema.Union(IPv4CidrMask, IPv6CidrMask)
}) {
  /**
   * The address family of this cidr block.
   *
   * @since 1.0.0
   */
  family = this.ip.family;
  /**
   * The first address in the range given by this address' subnet, often
   * referred to as the Network Address.
   *
   * @since 1.0.0
   */
  get networkAddressAsBigint() {
    const bits = this.family === "ipv4" ? 32 : 128;
    const bigIntegerAddress = this.ip.asBigint;
    const intermediate = bigIntegerAddress.toString(2).padStart(bits, "0").slice(0, this.mask);
    const networkAddressString = intermediate + "0".repeat(bits - this.mask);
    const networkAddressBigInt = BigInt(`0b${networkAddressString}`);
    return networkAddressBigInt;
  }
  /**
   * The first address in the range given by this address' subnet, often
   * referred to as the Network Address.
   *
   * @since 1.0.0
   */
  get networkAddress() {
    return this.family === "ipv4" ? IPv4.FromBigint(this.networkAddressAsBigint) : IPv6.FromBigint(this.networkAddressAsBigint);
  }
  /**
   * The last address in the range given by this address' subnet, often
   * referred to as the Broadcast Address.
   *
   * @since 1.0.0
   */
  get broadcastAddressAsBigint() {
    const bits = this.family === "ipv4" ? 32 : 128;
    const bigIntegerAddress = this.ip.asBigint;
    const intermediate = bigIntegerAddress.toString(2).padStart(bits, "0").slice(0, this.mask);
    const broadcastAddressString = intermediate + "1".repeat(bits - this.mask);
    const broadcastAddressBigInt = BigInt(`0b${broadcastAddressString}`);
    return broadcastAddressBigInt;
  }
  /**
   * The last address in the range given by this address' subnet, often
   * referred to as the Broadcast Address.
   *
   * @since 1.0.0
   */
  get broadcastAddress() {
    return this.family === "ipv4" ? IPv4.FromBigint(this.broadcastAddressAsBigint) : IPv6.FromBigint(this.broadcastAddressAsBigint);
  }
  /**
   * A stream of all addresses in the range given by this address' subnet.
   *
   * @since 1.0.0
   */
  get range() {
    const minValue = this.networkAddressAsBigint;
    const maxValue = this.broadcastAddressAsBigint;
    const addressFromBigint = this.family === "ipv4" ? IPv4.FromBigint : IPv6.FromBigint;
    const stream = Function.pipe(Stream.iterate(minValue, x => x + 1n), Stream.takeWhile(n => n <= maxValue), Stream.mapEffect(addressFromBigint));
    return stream;
  }
  /**
   * The total number of addresses in the range given by this address' subnet.
   *
   * @since 1.0.0
   */
  get total() {
    const minValue = this.networkAddressAsBigint;
    const maxValue = this.broadcastAddressAsBigint;
    return maxValue - minValue + 1n;
  }
}
/**
 * A schema that transforms a `string` into a `CidrBlock`.
 *
 * @since 1.0.0
 * @category Schemas
 */
exports.CidrBlock = CidrBlock;
const CidrBlockFromString = exports.CidrBlockFromString = /*#__PURE__*/Schema.transform(Schema.TemplateLiteral(Schema.String, Schema.Literal("/"), Schema.Number), CidrBlock, {
  decode: str => {
    const [ip, mask] = splitLiteral(str, "/");
    return {
      ip,
      mask: Number.parseInt(mask, 10)
    };
  },
  encode: ({
    ip,
    mask
  }) => `${ip}/${mask}`
}).annotations({
  identifier: "CidrBlockFromString",
  description: "A cidr block"
});
/**
 * An IPv4 wireguard endpoint, which consists of an IPv4 address followed by a
 * nat port then an optional local port. If only one port is provided, it is
 * assumed that the nat port and listen port are the same.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { IPv4Endpoint } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeEndpoint = Schema.decodeSync(IPv4Endpoint);
 *     const endpoint1 = decodeEndpoint("1.2.3.4:51820");
 *     const endpoint2 = decodeEndpoint("1.2.3.4:51820:41820");
 *
 *     const endpoint3 = decodeEndpoint({
 *         ip: "1.2.3.4",
 *         port: 51820,
 *     });
 *
 *     const endpoint4 = decodeEndpoint({
 *         ip: "1.2.3.4",
 *         natPort: 51820,
 *         listenPort: 41820,
 *     });
 */
const IPv4Endpoint = exports.IPv4Endpoint = /*#__PURE__*/Schema.transform(Schema.Union(Schema.Struct({
  ip: Schema.String,
  port: Schema.Number
}), Schema.Struct({
  ip: Schema.String,
  natPort: Schema.Number,
  listenPort: Schema.Number
}), Schema.TemplateLiteral(Schema.String, Schema.Literal(":"), Schema.Number), Schema.TemplateLiteral(Schema.String, Schema.Literal(":"), Schema.Number, Schema.Literal(":"), Schema.Number)), Schema.Struct({
  address: IPv4FromString,
  natPort: Port,
  listenPort: Port
}), {
  decode: data => {
    const isObjectInput = !Predicate.isString(data);
    const [ip, natPort, listenPort] = isObjectInput ? [data.ip, "natPort" in data ? `${data.natPort}` : `${data.port}`, "listenPort" in data ? `${data.listenPort}` : undefined] : splitLiteral(data, ":");
    const natPortParsed = Number.parseInt(natPort, 10);
    const listenPortParsed = Predicate.isNotUndefined(listenPort) ? Number.parseInt(listenPort, 10) : natPortParsed;
    return {
      address: ip,
      natPort: natPortParsed,
      listenPort: listenPortParsed
    };
  },
  encode: ({
    address,
    listenPort,
    natPort
  }) => `${address}:${natPort}:${listenPort}`
}).annotations({
  identifier: "IPv4Endpoint",
  description: "An ipv4 wireguard endpoint"
});
/**
 * An IPv6 wireguard endpoint, which consists of an IPv6 address in square
 * brackets followed by a nat port then an optional local port. If only one port
 * is provided, it is assumed that the nat port and listen port are the same.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { IPv6Endpoint } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeEndpoint = Schema.decodeSync(IPv6Endpoint);
 *     const endpoint1 = decodeEndpoint(
 *         "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:51820"
 *     );
 *     const endpoint2 = decodeEndpoint(
 *         "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:51820:41820"
 *     );
 *
 *     const endpoint3 = decodeEndpoint({
 *         ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
 *         port: 51820,
 *     });
 *
 *     const endpoint4 = decodeEndpoint({
 *         ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
 *         natPort: 51820,
 *         listenPort: 41820,
 *     });
 */
const IPv6Endpoint = exports.IPv6Endpoint = /*#__PURE__*/Schema.transform(Schema.Union(Schema.Struct({
  ip: Schema.String,
  port: Schema.Number
}), Schema.Struct({
  ip: Schema.String,
  natPort: Schema.Number,
  listenPort: Schema.Number
}), Schema.TemplateLiteral(Schema.Literal("["), Schema.String, Schema.Literal("]"), Schema.Literal(":"), Schema.Number), Schema.TemplateLiteral(Schema.Literal("["), Schema.String, Schema.Literal("]"), Schema.Literal(":"), Schema.Number, Schema.Literal(":"), Schema.Number)), Schema.Struct({
  address: IPv6FromString,
  natPort: Port,
  listenPort: Port
}), {
  decode: data => {
    const isObjectInput = !Predicate.isString(data);
    const tail = elements => elements.slice(1);
    const [ip, natPort, listenPort] = isObjectInput ? [data.ip, "natPort" in data ? `${data.natPort}` : `${data.port}`, "listenPort" in data ? `${data.listenPort}` : undefined] : [splitLiteral(data, "]")[0].slice(1), ...tail(splitLiteral(splitLiteral(data, "]")[1], ":"))];
    const natPortParsed = Number.parseInt(natPort, 10);
    const listenPortParsed = Predicate.isNotUndefined(listenPort) ? Number.parseInt(listenPort, 10) : natPortParsed;
    return {
      address: ip.slice(1),
      natPort: natPortParsed,
      listenPort: listenPortParsed
    };
  },
  encode: ({
    address,
    listenPort,
    natPort
  }) => `[${address}]:${natPort}:${listenPort}`
}).annotations({
  identifier: "IPv6Endpoint",
  description: "An ipv6 wireguard endpoint"
});
/**
 * A hostname wireguard endpoint, which consists of a hostname followed by a\
 * Nat port then an optional local port. If only one port is provided, it is
 * assumed that the nat port and listen port are the same.
 *
 * @since 1.0.0
 * @category Schemas
 */
const HostnameEndpoint = exports.HostnameEndpoint = /*#__PURE__*/Schema.transform(Schema.Union(Schema.Struct({
  host: Schema.String,
  port: Schema.Number
}), Schema.Struct({
  host: Schema.String,
  natPort: Schema.Number,
  listenPort: Schema.Number
}), Schema.TemplateLiteral(Schema.String, Schema.Literal(":"), Schema.Number), Schema.TemplateLiteral(Schema.String, Schema.Literal(":"), Schema.Number, Schema.Literal(":"), Schema.Number)), Schema.Struct({
  host: Schema.String,
  natPort: Port,
  listenPort: Port
}), {
  decode: data => {
    const isObjectInput = !Predicate.isString(data);
    const [host, natPort, listenPort] = isObjectInput ? [data.host, "natPort" in data ? `${data.natPort}` : `${data.port}`, "listenPort" in data ? `${data.listenPort}` : undefined] : splitLiteral(data, ":");
    const natPortParsed = Number.parseInt(natPort, 10);
    const listenPortParsed = Predicate.isNotUndefined(listenPort) ? Number.parseInt(listenPort, 10) : natPortParsed;
    return {
      host,
      natPort: natPortParsed,
      listenPort: listenPortParsed
    };
  },
  encode: ({
    host,
    listenPort,
    natPort
  }) => `${host}:${natPort}:${listenPort}`
}).annotations({
  identifier: "HostnameEndpoint",
  description: "A hostname endpoint"
});
/**
 * A wireguard endpoint, which is either an IPv4 or IPv6 endpoint.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *
 *     import { Endpoint } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeEndpoint = Schema.decodeSync(Endpoint);
 *     const endpoint1 = decodeEndpoint("1.2.3.4:51820");
 *     const endpoint2 = decodeEndpoint("1.2.3.4:51820:41820");
 *
 *     const endpoint3 = decodeEndpoint({
 *         ip: "1.2.3.4",
 *         port: 51820,
 *     });
 *
 *     const endpoint4: Endpoint = decodeEndpoint({
 *         ip: "1.2.3.4",
 *         natPort: 51820,
 *         listenPort: 41820,
 *     });
 *
 *     const endpoint5 = decodeEndpoint(
 *         "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:51820"
 *     );
 *     const endpoint6 = decodeEndpoint(
 *         "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]:51820:41820"
 *     );
 *
 *     const endpoint7 = decodeEndpoint({
 *         ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
 *         port: 51820,
 *     });
 *
 *     const endpoint8: Endpoint = decodeEndpoint({
 *         ip: "2001:0db8:85a3:0000:0000:8a2e:0370:7334",
 *         natPort: 51820,
 *         listenPort: 41820,
 *     });
 *
 * @see {@link IPv4Endpoint}
 * @see {@link IPv6Endpoint}
 */
const Endpoint = exports.Endpoint = /*#__PURE__*/Schema.Union(IPv4Endpoint, IPv6Endpoint, HostnameEndpoint).annotations({
  identifier: "Endpoint",
  description: "An ipv4 or ipv6 wireguard endpoint"
});
/**
 * A wireguard setup data, which consists of an endpoint followed by an address.
 *
 * @since 1.0.0
 * @category Schemas
 * @example
 *     import * as Schema from "@effect/schema/Schema";
 *     import { SetupData } from "the-wireguard-effect/InternetSchemas";
 *
 *     const decodeSetupData = Schema.decodeSync(SetupData);
 *     const setupData = decodeSetupData(["1.1.1.1:51280", "10.0.0.1"]);
 *
 * @see {@link EndpointSchema}
 * @see {@link Address}
 */
const SetupData = exports.SetupData = /*#__PURE__*/Schema.Tuple(Endpoint, AddressFromString).annotations({
  identifier: "SetupData",
  description: "A wireguard setup data"
});
//# sourceMappingURL=InternetSchemas.js.map