"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateStarConfigs = exports.generateP2PConfigs = exports.generateHubSpokeConfigs = exports.generate = exports.fromConfigFile = exports.WireguardSetConfigResolver = exports.WireguardSetConfigRequest = exports.WireguardIniConfig = exports.WireguardGetConfigResponse = exports.WireguardGetConfigResolver = exports.WireguardGetConfigRequest = exports.WireguardConfig = void 0;
var FileSystem = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform/FileSystem"));
var Path = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/platform/Path"));
var ParseResult = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/schema/ParseResult"));
var Schema = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("@effect/schema/Schema"));
var Array = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Array"));
var Chunk = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Chunk"));
var Effect = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Effect"));
var Either = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Either"));
var Function = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Function"));
var HashMap = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/HashMap"));
var Match = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Match"));
var Number = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Number"));
var Option = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Option"));
var Predicate = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Predicate"));
var Request = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Request"));
var Resolver = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/RequestResolver"));
var Sink = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Sink"));
var Stream = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Stream"));
var String = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/String"));
var Tuple = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("effect/Tuple"));
var ini = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("ini"));
var InternetSchemas = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./InternetSchemas.js"));
var WireguardControl = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardControl.js"));
var WireguardErrors = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardErrors.js"));
var WireguardInterface = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardInterface.js"));
var WireguardKey = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardKey.js"));
var WireguardPeer = /*#__PURE__*/_interopRequireWildcard( /*#__PURE__*/require("./WireguardPeer.js"));
function _getRequireWildcardCache(e) {
  if ("function" != typeof WeakMap) return null;
  var r = new WeakMap(),
    t = new WeakMap();
  return (_getRequireWildcardCache = function (e) {
    return e ? t : r;
  })(e);
}
function _interopRequireWildcard(e, r) {
  if (!r && e && e.__esModule) return e;
  if (null === e || "object" != typeof e && "function" != typeof e) return {
    default: e
  };
  var t = _getRequireWildcardCache(r);
  if (t && t.has(e)) return t.get(e);
  var n = {
      __proto__: null
    },
    a = Object.defineProperty && Object.getOwnPropertyDescriptor;
  for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) {
    var i = a ? Object.getOwnPropertyDescriptor(e, u) : null;
    i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u];
  }
  return n.default = e, t && t.set(e, n), n;
}
/**
 * Wireguard config schema definitions
 *
 * @since 1.0.0
 */

/**
 * A wireguard configuration.
 *
 * @since 1.0.0
 * @category Schemas
 */
class WireguardConfig extends Schema.Class("WireguardIniConfig")({
  /** TODO: Document */
  Address: InternetSchemas.CidrBlockFromString,
  /** TODO: Document */
  Dns: Schema.optional(InternetSchemas.AddressFromString),
  /**
   * The value for this is a decimal-string integer corresponding to the
   * listening port of the interface.
   */
  ListenPort: Schema.Union(InternetSchemas.Port, Schema.transformOrFail(Schema.String, InternetSchemas.Port, {
    decode: (str, _options, ast) => Either.fromOption(Number.parse(str), () => new ParseResult.Type(ast, str)),
    encode: port => Effect.succeed(`${port}`)
  })),
  /**
   * The value for this is a decimal-string integer corresponding to the
   * fwmark of the interface. The value may 0 in the case of a set operation,
   * in which case it indicates that the fwmark should be removed.
   */
  FirewallMark: Schema.optional(Schema.Number, {
    nullable: true
  }),
  /**
   * The value for this key should be a lowercase hex-encoded private key of
   * the interface. The value may be an all zero string in the case of a set
   * operation, in which case it indicates that the private key should be
   * removed.
   */
  PrivateKey: WireguardKey.WireguardKey,
  /** List of peers to add. */
  Peers: Schema.optional(Schema.Array(WireguardPeer.WireguardPeer), {
    default: () => [],
    nullable: true
  })
}) {
  /**
   * Writes a wireguard interface configuration to an INI file.
   *
   * @since 1.0.0
   * @category Constructors
   * @param file - The path to the INI file.
   */
  writeToFile = file => Effect.gen(this, function* () {
    const path = yield* Path.Path;
    const fs = yield* FileSystem.FileSystem;
    const configEncoded = yield* Schema.encode(WireguardConfig)(this);
    const iniConfigDecoded = yield* Schema.decode(WireguardIniConfig)(configEncoded);
    yield* fs.makeDirectory(path.dirname(file), {
      recursive: true
    });
    yield* fs.writeFileString(file, iniConfigDecoded);
  });
  /**
   * Starts a wireguard tunnel that will continue to run and serve traffic
   * even after the nodejs process exits.
   *
   * @since 1.0.0
   * @category Wireguard
   */
  up = interfaceObject => Effect.gen(this, function* () {
    const io = yield* Function.pipe(interfaceObject, Option.fromNullable, Option.map(Effect.succeed), Option.getOrElse(() => WireguardInterface.WireguardInterface.getNextAvailableInterface));
    const wireguardControl = yield* WireguardControl.WireguardControl;
    return yield* wireguardControl.up(this, io);
  });
  /**
   * Starts a wireguard tunnel that will be gracefully shutdown and stop
   * serving traffic once the scope is closed.
   *
   * @since 1.0.0
   * @category Wireguard
   */
  upScoped = interfaceObject => Effect.gen(this, function* () {
    const io = yield* Function.pipe(interfaceObject, Option.fromNullable, Option.map(Effect.succeed), Option.getOrElse(() => WireguardInterface.WireguardInterface.getNextAvailableInterface));
    const wireguardControl = yield* WireguardControl.WireguardControl;
    return yield* wireguardControl.upScoped(this, io);
  });
}
/**
 * A wireguard configuration encoded in the INI format.
 *
 * @since 1.0.0
 * @category Schema Transformations
 * @see {@link WireguardConfig}
 */
exports.WireguardConfig = WireguardConfig;
const WireguardIniConfig = exports.WireguardIniConfig = /*#__PURE__*/Schema.transformOrFail(WireguardConfig, Schema.String, {
  // Encoding is non-trivial, as we need to handle all the peers individually.
  decode: (config, _options, _ast) => Effect.gen(function* () {
    const listenPort = `ListenPort = ${config.ListenPort}\n`;
    const privateKey = `PrivateKey = ${config.PrivateKey}\n`;
    const address = `Address = ${config.Address.ip.ip}/${config.Address.mask}\n`;
    const dns = Predicate.isNotUndefined(config.Dns) ? `Dns = ${config.Dns?.ip}\n` : "";
    const fwmark = Predicate.isNotUndefined(config.FirewallMark) ? `FirewallMark = ${config.FirewallMark}\n` : "";
    const peersConfig = yield* Function.pipe(config.Peers, Array.map(peer => Schema.encode(WireguardPeer.WireguardPeer)(peer)), Array.map(Effect.flatMap(Schema.decode(WireguardPeer.WireguardIniPeer))), Effect.allWith(), Effect.map(Array.join("\n")));
    return `[Interface]\n${dns}${listenPort}${fwmark}${address}${privateKey}\n${peersConfig}`;
  }).pipe(Effect.mapError(({
    error
  }) => error)),
  // Decoding is likewise non-trivial, as we need to parse all the peers from the ini config.
  encode: (iniConfig, _options, _ast) => Effect.gen(function* () {
    const sections = iniConfig.split(/(?=\[Peer\])/g);
    const maybeInterfaceSection = Array.findFirst(sections, text => text.startsWith("[Interface]"));
    const interfaceSection = Option.getOrThrowWith(maybeInterfaceSection, () => new WireguardErrors.WireguardError({
      message: "No [Interface] section found"
    }));
    const peerSections = Function.pipe(sections, Array.filter(text => text.startsWith("[Peer]")), Array.map(text => text.replace("[Peer]", "")));
    const parsePeers = yield* Function.pipe(peerSections, Array.map(peer => Schema.encode(WireguardPeer.WireguardIniPeer)(peer)), Effect.allWith());
    const parseInterface = Function.pipe(interfaceSection, ini.parse, jsonConfig => ({
      ...jsonConfig["Interface"],
      Peers: parsePeers
    }), ({
      Address,
      Dns,
      FirewallMark,
      ListenPort,
      Peers,
      PrivateKey
    }) => ({
      Dns,
      Peers,
      Address,
      PrivateKey,
      ListenPort,
      FirewallMark: Number.parse(FirewallMark || "").pipe(Option.getOrUndefined)
    }), Schema.decode(WireguardConfig));
    return yield* parseInterface;
  }).pipe(Effect.mapError(({
    error
  }) => error))
}).annotations({
  identifier: "WireguardIniConfig",
  description: "A wireguard ini configuration"
});
/**
 * Generates two wireguard configurations, each with the other as a single peer
 * and shares their keys appropriately.
 *
 * @since 1.0.0
 * @category Constructors
 * @param aliceData - The data for the first peer.
 * @param bobData - The data for the second peer.
 */
const generateP2PConfigs = options => {
  const hub = options.aliceData;
  const spokes = Array.make(options.bobData);
  const configs = Predicate.isUndefined(options.cidrBlock) ? generate({
    preshareKeys: "generate",
    trustMap: "trustAllPeers",
    hubData: hub,
    spokeData: spokes
  }) : generate({
    preshareKeys: "generate",
    trustMap: "trustAllPeers",
    hubData: hub,
    spokeData: spokes,
    cidrBlock: options.cidrBlock,
    addressStartingIndex: options.addressStartingIndex
  });
  return Effect.map(configs, ([aliceConfig, [bobConfig]]) => Tuple.make(aliceConfig, bobConfig));
};
/**
 * Generates a collection of wireguard configurations for a star network with a
 * single central hub node and many peers all connected to it where the peers
 * all trust each other.
 *
 * @since 1.0.0
 * @category Constructors
 * @param hubData - The data for the hub node.
 * @param spokeData - The data for the spoke nodes.
 */
exports.generateP2PConfigs = generateP2PConfigs;
const generateStarConfigs = options => {
  const [firstNode, secondNode, ...rest] = options.nodes;
  return Predicate.isUndefined(options.cidrBlock) ? generate({
    preshareKeys: "generate",
    trustMap: "trustAllPeers",
    hubData: firstNode,
    spokeData: [secondNode, ...rest]
  }) : generate({
    preshareKeys: "generate",
    trustMap: "trustAllPeers",
    cidrBlock: options.cidrBlock,
    addressStartingIndex: options.addressStartingIndex,
    hubData: firstNode,
    spokeData: [secondNode, ...rest]
  });
};
/**
 * Generates a collection of wireguard configurations for a hub and spoke
 * network with a single central hub node and many peers all connected to it
 * where none of the peers trust each other.
 *
 * @since 1.0.0
 * @category Constructors
 * @param hubData - The data for the hub node.
 * @param spokeData - The data for the spoke nodes.
 */
exports.generateStarConfigs = generateStarConfigs;
const generateHubSpokeConfigs = options => {
  return Predicate.isUndefined(options.cidrBlock) ? generate({
    trustMap: "trustNoPeers",
    preshareKeys: "generate",
    hubData: options.hubData,
    spokeData: options.spokeData
  }) : generate({
    cidrBlock: options.cidrBlock,
    trustMap: "trustNoPeers",
    preshareKeys: "generate",
    addressStartingIndex: options.addressStartingIndex,
    hubData: options.hubData,
    spokeData: options.spokeData
  });
};
/**
 * Generates a collection of wireguard configurations.
 *
 * @since 1.0.0
 * @category Constructors
 */
exports.generateHubSpokeConfigs = generateHubSpokeConfigs;
const generate = options => Effect.gen(function* () {
  const inputIsSetupData = Array.isArray(options.hubData);
  const ipsNeeded = Array.length(options.spokeData) + 1;
  // Bounds checking on the cidr block
  if (options.cidrBlock && options.cidrBlock.total < ipsNeeded) {
    return yield* new WireguardErrors.WireguardError({
      message: `Not enough IPs in the CIDR block for ${ipsNeeded} nodes`
    });
  }
  // Generate some ip addresses to use if the input data was just an endpoint
  const ips = options.cidrBlock?.range ? yield* Function.pipe(options.cidrBlock?.range, Stream.drop(options.addressStartingIndex ?? 0), Stream.mapEffect(Schema.encode(InternetSchemas.Address)), Stream.run(Sink.collectAllN(ipsNeeded)), Effect.map(Chunk.toArray)) : yield* Effect.succeed(Array.empty());
  const trustMap = Function.pipe(Match.value(options.trustMap), Match.when("trustAllPeers", () => Function.pipe(options.spokeData, Array.map(spoke => Tuple.make(spoke, options.spokeData)), HashMap.fromIterable)), Match.when(HashMap.isHashMap, Function.identity), Match.whenOr("trustNoPeers", Predicate.isUndefined, () => Function.unsafeCoerce(HashMap.empty())), Match.exhaustive);
  const preshareKeys = Function.pipe(Match.value(options.preshareKeys), Match.when("generate", () => Function.pipe(options.spokeData, Array.map(spoke => Tuple.make(spoke, WireguardKey.generatePreshareKey())), HashMap.fromIterable)), Match.when(HashMap.isHashMap, Function.identity), Match.when(Predicate.isUndefined, () => Function.unsafeCoerce(HashMap.empty())), Match.exhaustive);
  // Convert all inputs to be SetupDataEncoded
  const hubSetupDataEncoded = inputIsSetupData ? options.hubData : Tuple.make(options.hubData, ips.at(0)?.ip ?? "");
  const spokeSetupDataEncoded = inputIsSetupData ? options.spokeData : Array.map(options.spokeData, (spoke, index) => Tuple.make(spoke, ips.at(index + 1)?.ip ?? ""));
  // Decode all SetupData inputs
  const hubSetupData = yield* Schema.decode(InternetSchemas.SetupData)(hubSetupDataEncoded);
  const spokeSetupData = yield* Effect.all(Array.map(spokeSetupDataEncoded, spoke => Schema.decode(InternetSchemas.SetupData)(spoke)));
  const spokeSetupDataBoth = Array.zip(spokeSetupDataEncoded, spokeSetupData);
  // Generate the keys for the hub
  const hubKeys = WireguardKey.generateKeyPair();
  const hubPreshareKey = HashMap.get(preshareKeys, hubSetupDataEncoded).pipe(Option.getOrUndefined);
  // This hub peer config will be added to all the spoke interface configs
  const hubPeerConfig = {
    PresharedKey: hubPreshareKey,
    PublicKey: hubKeys.publicKey,
    Endpoint: Tuple.getFirst(hubSetupDataEncoded),
    AllowedIPs: new Set([`${Tuple.getSecond(hubSetupDataEncoded)}/32`])
  };
  // All these spoke peer configs will be added to the hub interface config
  const spokePeerConfigs = Array.map(spokeSetupDataBoth, ([spokeEncoded, spokeDecoded]) => {
    const keys = WireguardKey.generateKeyPair();
    const preshareKey = HashMap.get(preshareKeys, spokeEncoded).pipe(Option.getOrUndefined);
    return {
      setupDataEncoded: spokeEncoded,
      setupDataDecoded: spokeDecoded,
      keys: {
        preshareKey,
        privateKey: keys.privateKey
      },
      peerConfig: {
        PresharedKey: preshareKey,
        PublicKey: keys.publicKey,
        Endpoint: Tuple.getFirst(spokeEncoded),
        AllowedIPs: new Set([`${Tuple.getSecond(spokeEncoded)}/32`])
      }
    };
  });
  // The hub will get all the peers added to it
  const spokePeerConfigsBySetupData = Function.pipe(spokePeerConfigs, Array.map(peer => Tuple.make(peer.setupDataEncoded, peer)), HashMap.fromIterable);
  // The hub interface config will have all the spoke peer configs
  const hubConfig = yield* Schema.decode(WireguardConfig)({
    PrivateKey: hubKeys.privateKey,
    ListenPort: Tuple.getFirst(hubSetupData).listenPort,
    Peers: Array.map(spokePeerConfigs, ({
      peerConfig
    }) => peerConfig),
    Address: `${Tuple.getSecond(hubSetupDataEncoded)}/${options.cidrBlock?.mask ?? 24}`
  });
  // Each spoke interface config will have the hub peer config and the other peers from the trust map
  const spokeConfigs = yield* Function.pipe(spokePeerConfigs, Array.map(({
    keys: {
      privateKey
    },
    setupDataDecoded,
    setupDataEncoded
  }) => {
    const friends = Function.pipe(trustMap, HashMap.get(setupDataEncoded), Option.getOrElse(() => Array.empty()), Array.map(friend => HashMap.get(spokePeerConfigsBySetupData, friend)), Array.map(Option.getOrThrow), Array.map(({
      peerConfig
    }) => peerConfig));
    return Schema.decode(WireguardConfig)({
      PrivateKey: privateKey,
      Peers: [hubPeerConfig, ...friends],
      ListenPort: Tuple.getFirst(setupDataDecoded).listenPort,
      Address: `${Tuple.getSecond(setupDataEncoded)}/${options.cidrBlock?.mask ?? 24}`
    });
  }), Effect.allWith());
  return Tuple.make(hubConfig, spokeConfigs);
});
/**
 * Loads a wireguard interface configuration from an INI file.
 *
 * @since 1.0.0
 * @category Constructors
 * @param file - The path to the INI file.
 */
exports.generate = generate;
const fromConfigFile = file => Effect.gen(function* () {
  const fs = yield* FileSystem.FileSystem;
  const fsConfig = yield* fs.readFileString(file);
  const iniConfigEncoded = yield* Schema.encode(WireguardIniConfig)(fsConfig);
  const config = yield* Schema.decode(WireguardConfig)(iniConfigEncoded);
  return config;
});
/**
 * @since 1.0.0
 * @category Responses
 * @see https://www.wireguard.com/xplatform/
 */
exports.fromConfigFile = fromConfigFile;
const WireguardGetConfigResponse = exports.WireguardGetConfigResponse = /*#__PURE__*/Function.pipe(WireguardConfig, Schema.omit("Peers"), Schema.extend(Schema.Struct({
  Peers: Schema.optional(Schema.Array(WireguardPeer.WireguardUApiGetPeerResponse), {
    default: () => [],
    nullable: true
  })
}))).annotations({
  identifier: "WireguardGetConfigResponse",
  description: "The response of a WireguardGetConfigRequest"
});
/**
 * @since 1.0.0
 * @category Requests
 */
class WireguardGetConfigRequest extends Request.TaggedClass("WireguardGetConfigRequest") {}
/**
 * @since 1.0.0
 * @category Requests
 */
exports.WireguardGetConfigRequest = WireguardGetConfigRequest;
class WireguardSetConfigRequest extends Request.TaggedClass("WireguardSetConfigRequest") {}
/**
 * @since 1.0.0
 * @category Resolvers
 */
exports.WireguardSetConfigRequest = WireguardSetConfigRequest;
const WireguardGetConfigResolver = exports.WireguardGetConfigResolver = /*#__PURE__*/Resolver.fromEffect(({
  address,
  wireguardInterface
}) => Effect.gen(function* () {
  const uapiConfig = yield* WireguardControl.userspaceContact(wireguardInterface, "get=1\n\n");
  const [interfaceConfig, ...peers] = uapiConfig.split("public_key=");
  const {
    fwmark,
    listen_port,
    private_key
  } = ini.decode(interfaceConfig);
  const peerConfigs = yield* Function.pipe(peers, Array.map(peer => `public_key=${peer}`), Array.map(peer => WireguardPeer.parseWireguardUApiGetPeerResponse(peer)), Array.map(Effect.flatMap(Schema.encode(WireguardPeer.WireguardUApiGetPeerResponse))), Effect.allWith());
  return yield* Schema.decode(WireguardGetConfigResponse)({
    Address: address,
    ListenPort: listen_port,
    PrivateKey: Buffer.from(private_key, "hex").toString("base64"),
    FirewallMark: Number.parse(fwmark || "").pipe(Option.getOrUndefined),
    Peers: peerConfigs
  });
}));
/**
 * @since 1.0.0
 * @category Resolvers
 */
const WireguardSetConfigResolver = exports.WireguardSetConfigResolver = /*#__PURE__*/Resolver.fromEffect(({
  config,
  wireguardInterface
}) => Effect.gen(function* () {
  const listenPort = `listen_port=${config.ListenPort}\n`;
  const privateKeyHex = Buffer.from(config.PrivateKey, "base64").toString("hex");
  const privateKey = `private_key=${privateKeyHex}\n`;
  const fwmark = Predicate.isNotUndefined(config.FirewallMark) ? `fwmark=${config.FirewallMark}\n` : String.empty;
  const peers = Function.pipe(config.Peers, Array.map(peer => WireguardPeer.makeWireguardUApiSetPeerRequest(peer)), Array.join("\n"));
  const uapiConfig = `${privateKey}${listenPort}${fwmark}${peers}\n`;
  yield* WireguardControl.userspaceContact(wireguardInterface, `set=1\n${uapiConfig}\n`);
  return wireguardInterface;
}));
//# sourceMappingURL=WireguardConfig.js.map